library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.axi3intercon_pkg.all;
use work.axi_aw_router_pkg.all;

entity axi3intercon_aw_router is
	port(
		aclk           : in  std_logic;
		rst            : in  std_logic;
		masters_out    : in  axi_aw_masters_out_t(0 to MASTER_COUNT - 1);
		masters_in     : out axi_aw_masters_in_t(0 to MASTER_COUNT - 1);
		slaves_out     : in  axi_aw_slaves_out_t(0 to SLAVE_COUNT);
		slaves_in      : out axi_aw_slaves_in_t(0 to SLAVE_COUNT);
		write_locks    : out write_locks_t(0 to MASTER_COUNT - 1); -- write_* signals come/go from/to the write router.
		write_releases : in  write_release_t;
		address_array  : in  axi_slave_addresses_t(0 to SLAVE_COUNT - 1);
		mask_array     : in  axi_slave_addresses_t(0 to SLAVE_COUNT - 1)
	);
end entity axi3intercon_aw_router;

architecture RTL of axi3intercon_aw_router is
	alias clk is aclk;
	type aw_state_t is (READY, ACTIVE, BLK);
	type aw_states_t is array (0 to MASTER_COUNT - 1) of aw_state_t;
	signal aw_states : aw_states_t;
begin
	aw_router : process(clk, rst) is
		variable slave_block : std_logic_vector(0 to SLAVE_COUNT - 1) := (others => '0');
		variable slave_idx   : integer range 0 to SLAVE_COUNT         := SLAVE_COUNT; -- 1 more slave than connected for handling bad requests.

		procedure calculate_slave(address : in std_logic_vector(ADDRESS_BITS - 1 downto 0)) is
		begin
			slave_idx := SLAVE_COUNT;
			for i in 0 to SLAVE_COUNT - 1 loop
				if (address and mask_array(i)) = address_array(i) then
					slave_idx := i;
				end if;
			end loop;

		end procedure calculate_slave;

	begin
		if rst = '1' then
			for i in 0 to MASTER_COUNT - 1 loop
				aw_states(i) <= READY;
			end loop;
			slave_block := (others => '0');
		elsif rising_edge(clk) then
			for i in MASTER_COUNT - 1 downto 0 loop -- Loop for every master
				case aw_states(i) is
					when READY =>
						if masters_out(i).awvalid = '1' then
							calculate_slave(masters_out(i).awaddr);
						-- TODO: Write router itself
						-- TODO: lock output
						end if;
					when others => null;
				end case;

			end loop;
		end if;
	end process aw_router;

end architecture RTL;
